from __future__ import annotations

import typing

from .StiBaseResult import StiBaseResult

if typing.TYPE_CHECKING:
    from ..StiDataAdapter import StiDataAdapter
    from ..StiSqlAdapter import StiSqlAdapter


class StiDataResult(StiBaseResult):
    """
    The result of executing an event handler request. 
    The result contains a collection of data, message about the result of the command execution, and other technical information.
    """

### Properties

    adapterVersion: str = None
    types: list = None
    columns: list = None
    rows: list = None
    data: str = None
    dataType: str = None
    count = 0

    @property
    def type(self) -> str:
        if self.success:
            if hasattr(self, 'columns') and isinstance(self.columns, list):
                return 'SQL'

            if self.dataType != None:
                return 'File'

        return super().type


### Result

    def getDataAdapterResult(self, adapter: StiDataAdapter) -> StiDataResult:
        self.adapterVersion = adapter.version
        self.checkVersion = adapter.checkVersion

        from ..StiSqlAdapter import StiSqlAdapter
        if isinstance(adapter, StiSqlAdapter):
            self.types = []
            self.columns = []
            self.rows = []

        return self

    @staticmethod
    def getSuccess(notice: str = None) -> StiDataResult:
        """Creates a successful result."""
        
        result: StiDataResult = StiBaseResult.getSuccess(notice)
        result.__class__ = StiDataResult
        
        return result
    
    @staticmethod
    def getError(notice: str) -> StiDataResult:
        """Creates an error result."""

        result: StiDataResult = StiBaseResult.getError(notice)
        result.__class__ = StiDataResult
        
        return result


from urllib import response
import requests

from .classes.StiDataResult import StiDataResult
from .classes.StiPath import StiPath
from .enums.StiDataType import StiDataType
from .StiDataAdapter import StiDataAdapter


class StiFileAdapter(StiDataAdapter):
    
### Properties

    dataType = StiDataType.TEXT
    """The data type loaded by the data adapter."""

    connectionLink: StiPath = None
    """Link to the created database connection driver."""

### Methods

    def connect(self) -> StiDataResult:
        path = StiPath(self.connectionString)
        if path.filePath == None:
            return StiDataResult.getError(f"Data file '{self.connectionString}' not found.").getDataAdapterResult(self)

        self.connectionLink = path

        return StiDataResult.getSuccess().getDataAdapterResult(self)
    
    def getDataResult(self, filePath: str, maxDataRows: int = -1) -> StiDataResult:
        self.connectionString = filePath
        self.process()

        result = self.connect()
        if result.success:
            try:
                if self.connectionLink.fileUrl != None:
                    response = requests.get(self.connectionLink.fileUrl)
                    if response.status_code == 200:
                        result.data = response.content.decode()
                else:
                    with open(self.connectionLink.filePath, mode='r', encoding='utf-8') as file:
                        result.data = file.read()

                result.dataType = self.dataType.value
                
            except Exception as e:
                message = str(e)
                result = StiDataResult.getError(message).getDataAdapterResult(self)

        return result
    


class StiConnectionInfo:
    """All database connection information taken from the connection string."""

    driver = ''
    host = ''
    port = 0
    database = ''
    userId = ''
    password = ''
    charset = ''
    privilege = ''


from .classes.StiBaseHandler import StiBaseHandler
from .classes.StiBaseResult import StiBaseResult
from .classes.StiDataResult import StiDataResult
from .classes.StiBaseResponse import StiBaseResponse


from enum import Enum


class StiDataCommand(Enum):

    NONE = None
    GET_SUPPORTED_ADAPTERS = 'GetSupportedAdapters'
    GET_SCHEMA = 'GetSchema'
    GET_DATA = 'GetData'
    TEST_CONNECTION = 'TestConnection'
    RETRIEVE_SCHEMA = 'RetrieveSchema'
    EXECUTE = 'Execute'
    EXECUTE_QUERY = 'ExecuteQuery'


### Helpers

    @staticmethod
    def getValues():
        return [enum.value for enum in StiDataCommand if enum.value != None]